{{
ADXL345Object

Author: Andrew Colwell (c) February 2010
Version: 1.0

Please see description in the ADXL345Object for more details.

First, the code tests to see if the ADXL345 is present.

Second, the code runs a Self test.
The Self Test Data often fails, yet the chip seems to work fine. Its
been through some tumbles, and I don't have a second to further test.

Finally, the code reads the acceleration recorded on each axis in
decimal representing +/-16g in 10bits.

+/-16g means a range of 32g
32g / 2^10 bits = 0.03125g per bit

multiplying the value displayed on the Serial Terminal by 0.03125
will give the current g's the axis is experiencing. If the chip
is level, the x and y axes will be 0, and the z axis will be 32
representing 1g (32 * 0.03125 = 1). This is good as the z-axis
measures the vertical axis and if sitting still will experience
1g of acceleration due to gravity!

The arrows on the break out board indicate positive accelerations.

Changing the orientation of the breakout board by 90 will result
in another axis reading +/-32 and the z axis falling to 0.

At an orientation of 45 will result in 23 for two axis. Force is
a vector, and forms a right angle triange with the two axis as the
components. Pythagorean theorem can be used to solve for the hypotenuse.
32 = z + x    at 45 the z and x components will be equal
1024 = z+ z
1024 = 2*z
512 = z
z = 22.6         and the chip experiences 23 in digital
         
}}
CON
   _clkmode      = xtal1 + pll16x
   _xinfreq      = 5_000_000

  ' Register Map addresses (partial list)
   _DeviceID   = $00   'Device ID (0xE5)      r   Always %1110_0101
   _xOffset    = $1E   'User defined offset   r/w Each bit has a factor of 
   _yOffset    = $1F   'User defined offset   r/w 15.6mg/LSB per offset
   _zOffset    = $20   'User defined offset   r/w
   _FreeFallTh = $28   'freefall threshold    r/w 62.5mg/LSB Recommended between 0x05 and 0x09
   _FreeFall   = $29   'freefall time         r/w
   _Rate       = $2C   'Transfer Rate         r/w See datasheet table default 100Hz output
   _PwrCtrl    = $2D   'Measurement Controls  r/w
   _IntEnable  = $2E   'Interrupt control     r/w (%0000_0100 for freefall)
   _IntMap     = $2F   'Interrupt mapping     r/w (%0000_0000 for Int1 output)
   _IntSource  = $30   'Source of interrupts  r   (%0000_0100 freefall triggered int1)
   _DataFormat = $31   'Data format           r/w (%0000_0011 +/-16g with sign extension, 10 bit mode)
                       '                          (%1000_0000 Self-Test)
   _X0         = $32   '                      r    LSB
   _X1         = $33   '                      r    MSB
   _Y0         = $34   '                      r    LSB
   _Y1         = $35   '                      r    MSB
   _Z0         = $36   '                      r    LSB
   _Z1         = $37   '                      r    MSB
   _FifoCtrl   = $38   'FIFO control          r/w
   _FifoStat   = $39   'FIFO status           r
{{
   ACK      = 0        ' I2C Acknowledge
   NAK      = 1        ' I2C No Acknowledge
   Xmit     = 0        ' I2C Direction Transmit
   Recv     = 1        ' I2C Direction Receive
   BootPin  = 28       ' I2C Boot EEPROM SCL Pin
   EEPROM   = $A0      ' I2C EEPROM Device Address
   SCL      = 20       ' I2C SCL Pin
   SDA      = 21       ' I2C SDA Pin
}}

  No = 0                                                                                ' Enumeration of 'No'
  Yes =1                                                                                ' Enumeration of 'Yes'

OBJ
        uart                    :                       "FullDuplexSerial"                                       
        Debug                   :                       "Serial_Lcd_Devantech"            
        adxl                    :                       "ADXL345Object"                   
        sdfat                   :                       "fsrw2_6A"
        num                     :                       "Simple_Numbers"
        gps                     :                       "GPS_Float_Lite"
        FS                      :                       "FloatString"                                                                        
VAR

  long xaxis, yaxis, zaxis, temp, axis, axis0, axis1

  Byte dataIn[100] 
  Byte dataOut[100], r
  Byte Rx
  Byte AnsaA, AnsaB
  Byte H3, H4
  Byte ct  , err
  Byte lcdMode
  byte coolant, load, map, iat, rpm, speed, lambda, throttle, advance, afm                                                                
  byte stackOBD[35]
  byte ready
  
PUB Main    | trk, ctr, cr,B

  LcdMode :=0
  ready := 0


  
  GPS.Init                                              ' Start comms with GPS
  
  Debug.init(17, 9600, 2)                               ' Start comms with LCD                        

  uart.start(8, 9, 3, 238000)                           ' Start comms with ELM327 OBD reader
  'NB is a blocking receive
  
  uart.rxflush                                          ' Flush the receive buffer                        
   
  CallATZ                                               ' Reset the ELM327
  
  callCLS                                               ' Clear the LCD



'cR := cognew(OBD, @stackOBD)                          ' <============================================== STOPS HERE IF UNCOMMENTED



  DEBUG.dec(cR)

  
  debug.gotoXY(1,1)                                     ' Move cursor to top left  
  debug.str(string("Mounting SD..."))                   ' Tell the user what is happening


  r:=  sdfat.mount_explicit(2, 3, 4, 5)                 ' Mount the SD card

  callCLS
  debug.gotoXY(1,1)
  debug.str(string("OK..."))
  
  adxl.InitI2C                                          ' Initiate Accelerometers


  {{For changing the baud rate
    CALLBAUD
    WAITCNT(CNT+160000000)       

    CALLATZ
    WAITCNT(CNT+160000000)       

    BAUDB  
    WAITCNT(CNT+160000000)
   }}      
  
  adxl.WriteDataFormat(%0000_10000)                     '%0000_0000 is 2g, %0000_0001 is 4g, %0000_0010 is 8g, %0000_0011 is 16g
                                                        '%0000_1000 where 1 is 13 bit data (4mg/LSB) and 0 is 10 bit data
  adxl.WrLoc(_Rate, %0000_1011)                         'Set data rate at 100Hz
  adxl.WrLoc(_PwrCtrl, %0000_1000)                      'Set chip to take measurements


 callCLS 
   
startWrite                                              ' Open file on SD card

 repeat  30                                             ' Get 30 samples                        
 
    
    if trk > 5                                                                  ' Every fifth sample....
          
          sdfat.pflush                                                          '                        ...flush the buffer of the SD card                    
          sdfat.sdstr(FS.FloattoString(GPS.Float_Latitude_Deg))                 '                        ...write the latitude to the SD card
          SDcomma                                                               '                        ...write a comma to the SD card
          sdfat.sdstr(FS.FloattoString(GPS.Float_Longitude_Deg))                '                        ...write the longitude to the SD card
          SDcomma                                                               '                        ...write a comma to the SD card
          trk := 0
          
    else
    
          SDcomma                                                       ' Otherwise
          SDcomma                                                       '                        ...write a comma to the SD card
                                                                        '                        ...write a comma to the SD card
    
    sdfat.sddec(cnt)
    SDcomma                                                             '                        ...write a comma to the SD card
    sdfat.sddec((adxl.Read2byte(_X0)))                                  '                        ...write acceleration data - X    
    SDcomma                                                             '                        ...write a comma to the SD card
    sdfat.sddec((adxl.Read2byte(_Y0)))                                  '                        ...write acceleration data - Y
    SDcomma                                                             '                        ...write a comma to the SD card 
    sdfat.sddec((adxl.Read2byte(_Z0)))                                  '                        ...write acceleration data - Z
    SDcomma                                                             '                        ...write a comma to the SD card

    'if ready == 1
        'callcoolant                                    '
        'callwritestuff(1)
        
        callload
        callwritestuff(1)

        callMap
        callwritestuff(1)
        
        callRPM
        callwritestuff(1)
        
        'callspeed
        'callwritestuff(1)
        
        'calladvance
        'callWriteStuff(advance)          
        'SDcomma

        'calliat
        'callwritestuff(1)

        callafm
        callwritestuff(1)

        callthrottle
        callwritestuff(0)
        
        '
       
    SDCR
    trk++
    


adxl.WrLoc(_PwrCtrl, %0000_0000)            'Set chip to stop taking measurements   
sdfat.pflush
sdfat.unmount
debug.str(string("OK..."))

  

pub callWriteStuff(co)
  {{callWriteStuff contains a small amount of code that writes the answer from OBD data calculations to the SD card.}}
  bytemove(@dataOut,num.dec(AnsaA),5)                                                   ' Copy the answer to the buffer as a decimal
  sdfat.pwrite(@dataOut,5)                                                              ' Write the buffer (5 characters) to the SD file
  if co == 1
       SDcomma                                                                    ' Write the delimiter to the filePUB callWriteGPS


pub callCls
   debug.cls

pub startWrite
 
  callCls
  debug.str(string("Opening: "))                               
  sdfat.popen(string("TEST.csv"), "w")

PUB callWriteByte(v, l)
  bytemove(@dataOut,num.dec(v), l)                                                   ' Copy the answer to the buffer as a decimal
  sdfat.pwrite(@dataOut,5) 

pub OBD | a
  repeat
    a:= a                       ' just a test action
     {{ repeat until ready == 0                         ' Wait until data is requested
       
      callCoolant                                       ' Then get the data
      callLoad
      callMap
      callRPM
      callSpeed
      callAdvance
      callIAT
      callAFM
      callThrottle
      callLambda
    
       
      ready := 1                                         ' The main loop can deal with this}}

    
pub callAT
  {{callAT sends a string to the ELM327 that is "AT ". This is basic to many of the commands that need to be sent to the ELM327, so is a useful building block that can be re-used many times.}}
  uart.tx(65)                                                                           ' Send character 'A'
  uart.tx(84)                                                                           ' Send character 'T'
  uart.tx(32)                                                                           ' Send a space character  

pub callbaud
  {{CAn be used to change the baud rate}}
  uart.tx(65)'A                                                                           ' Send character 'A'
  uart.tx(84) 'T                                                                          ' Send character 'T'
  uart.tx(32) ' SPACE                                                                          ' Send a space character  
  uart.tx(80) 'p
  uart.tx(80) 'P
  uart.tx(32) 'SPACE
  uart.tx(48) '0
  uart.tx(67) 'C
  uart.tx(32) 'SPACE
  uart.tx(83) 'S
  uart.tx(86) 'V
  uart.tx(32) 'SPACE
  uart.tx(49) '2
  uart.tx(49) '2
  uart.tx(13) 
  CALLWAITRX
  
PUB BAUDb
  {{CAn be used to change the baud rate}}
  uart.tx(65)'A                                                                           ' Send character 'A'
  uart.tx(84) 'T                                                                          ' Send character 'T'
  uart.tx(32) ' SPACE                                                                          ' Send a space character  
  uart.tx(80) 'p
  uart.tx(80) 'P
  uart.tx(32) 'SPACE
  uart.tx(48) '0
  uart.tx(67) 'C
  uart.tx(32) 'SPACE
  uart.tx(79) '2
  uart.tx(78) '2
  uart.tx(13) 
  CALLWAITRX
  
pub callATZ
  {{callATZ sends a string to the ELM327 that is "AT Z". This resets the ELM327 and results in its descriptor being returned. We don't check for the descriptor being as expected.}}
   debug.str(string("Reset:"))
   callAT                                                                               ' Send characters "AT "
   uart.tx(90)                                                                          ' Send character 'Z'            
   uart.tx(13)                                                                          ' Send a carriage return
   callWaitRx                                                                           ' Wait for, and process, the returned data
   
pub callCoolant 
   {{callCoolant requests PID05 data - engine coolant temperature. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 53)                                                                    ' Send the request                                                                    
   callLCDShow (string("Coolant temp"),string(" C"), AnsaA - 40)                        ' Show the data on the LCD

pub callLoad     
   {{callLoad requests PID04 data - engine load. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 52)                                                                    ' Send the request
   callLCDShow(string("Engine Load"), string(" %"), AnsaA * 100 / 255 )                 ' Show the data on the LCD   
   
   AnsaA := AnsaA * 100/255 
pub callMAP     
   {{callMAP requests PID0B data - manifold pressure. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 66)                                                                    ' Send the request
   callLCDShow (string("MAP px"),string("kPa"), AnsaA * 100 / 255)                      ' Show the data on the LCD
  
pub callRPM            
   {{callRPM requests PID0C data - engine RPM. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 67)
   callLCDShow(string("Engine RPM"), string(" "), ((AnsaB  * 256) + AnsaA)/4)           ' Show the data on the LCD   
   
pub callSpeed      
   {{callRPM requests PID0D data - Vehicle speed. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 68)                                                                    ' Send the request
   callLCDShow (string("Speed"), string(" km/h"), AnsaA)                                ' Show the data on the LCD
   
pub callAdvance    
   {{callRPM requests PID0E data - Ignition advance. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 69)                                                                    ' Send the request
   callLCDShow(string("Advance"), string(" Deg"),AnsaA / 2 - 64 )                       ' Show the data on the LCD
   
pub callIAT    
   {{callRPM requests PID0F data - Inlet air temperaturee. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(48, 70)                                                                    ' Send the request
   callLCDShow (string("Inlet Temp"), string(" C"), AnsaA - 40)                         ' Show the data on the LCD
   
pub callAFM    
   {{callRPM requests PID10 data - Airflow rate. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(49, 48)                                                                    ' Send the request
   callLCDShow(string("Airflow"), string(" g/s"), ((AnsaB * 256)+ AnsaA)/100)           ' Show the data on the LCD
   
   AnsaA := ((AnsaB * 256)+ AnsaA)/100

pub callFuelPxPetrol
   callPID01(50, 50)                                                                    ' Send the request
   callLCDShow(string("Fuel Px(P)"), string(" kPa"), (((AnsaB * 256)+ AnsaA)/10)/128)           ' Show the data on the LCD
   AnsaA := (((AnsaB * 256)+ AnsaA)/10)/128

pub callFuelPxDiesel
   callPID01(50, 51)                                                                    ' Send the request
   callLCDShow(string("Fuel Px(D)"), string(" kPa"), (((AnsaB * 256)+ AnsaA)/10))           ' Show the data on the LCD
   AnsaA := (((AnsaB * 256)+ AnsaA)/10)

pub callBaro
   callPID01(52, 52)                                                                    ' Send the request
   callLCDShow(string("Baro Px(P)"), string(" kPa"), AnsaA)           ' Show the data on the LCD

pub callThrottle
   {{callThrottle requests PID49 data - Throttle position. The returned data is converted from the hex to decimal and displayed to the user on the LCD (or recorded).}}
   callPID01(52, 57)                                                                    ' Send the request
   callLCDShow(string("Throttle pos"), string(" %"), AnsaA * 100 / 255)        ' Show the data on the LCD
   AnsaA := AnsaA * 100 / 255
   
pub callWaitRx  
   {{callWaitRx receives a string of characters from the ELM327 unit. These are loaded into a buffer until the end of the data is detected. The data in the buffer is then available for processing and analysis.}}
   'callcls
   longfill(dataIn, 100, 0)                                                             ' Empty the dataIn variable
   longfill(dataOut, 100, 0)                                                            ' Empty the dataOut variable
   ct := 0                                                                              ' Reset the counter for the number of characters received
    repeat while Rx <>= 62                                                               ' Continue to collect data until the end of the data (ASCII 62)     
     Rx := uart.Rx                                                                      ' Get character from Rx Buffer
     dataIn[ct++] := Rx                                                                 ' Place the received character in the dataIn variable
     'debug.putc(Rx)                                                                      ' Echo characters received to screen
   Rx:=42                                                                               ' Set 'Rx' to a value that is NOT 62, otherwise it won't                                                                                        ' go through the 'repeat while' loop on next encounter                   uart.rxflush                                                                   ' Flush the Rx buffer
  

   
pub callPID01(cChar1, cChar2)
  {{callPID01 is a procedure that can call any Mode 1 PID, wait for the results and load the data into the Hexn variables for processing into the answer.}}

  uart.tx(48)                                                                           ' Send '0'
  uart.tx(49)                                                                           ' Send '1'
  uart.tx(32)                                                                           ' Send a space
  uart.tx(cChar1)                                                                       ' Send the first character of the PID ID
  uart.tx(cChar2)                                                                       ' Send the second character of the PID ID
  uart.tx(13)                                                                           ' Send a carriage return
            
  callWaitRx                                                                            ' Receive the data from the ELM327
  H3 := dataIn[ct - 6]                                                                  ' Populate the Hex3 variable with the penultimate character of the response.
  H4 := dataIn[ct - 5]                                                                  ' Populate the Hex4 variable with the last character of response.

  err:= 8
  if H3 == 84 and H4 == 65                                                               ' Capture 'NO DATA' error
    'callRetry(string("NO DATA"))         
  elseif H3 == 82 and H4 == 79    
    'callRetry(string("DATA ERR"))                                                        ' Capture 'DATA ERROR' error
  elseif H4 == 63
    'callRetry(string("WHAT?"))                                                           ' Capture error where request has been rejected by ELM327
  else
     err := 0  

  callhexToDec(dataIn[ct - 9], dataIn[ct - 8], dataIn[ct - 6], dataIn[ct - 5])            ' Convert the data to decimal.


     
pub callhexToDec (Hex1, Hex2, Hex3, Hex4) | V1, V2, V3, V4
{{callHexToDec converts fours ASCII-coded characters into decimal values}}
V1 := 0                                                                                 ' Set the value variables to 0
V2 := 0
V3 := 0
V4 := 0
 
if Hex1 > 64                                                                            ' Convert the ASCII code to decimal - character 1
   V1 := Hex1 - 55
else
   V1 := Hex1 - 48
 
if Hex2 > 64                                                                            ' Convert the ASCII code to decimal - character 2
   V2 := Hex2 - 55
else
   V2 := Hex2 - 48
 
if Hex3 > 64                                                                            ' Convert the ASCII code to decimal - character 3
   V3 := Hex3 - 55
else
   V3 := Hex3 - 48
 
if Hex4 > 64                                                                            ' Convert the ASCII code to decimal - character 4
   V4 := Hex4 - 55
else
   V4 := Hex4 - 48
 
AnsaA := (V3 * 16) + V4                                                                 ' Convert into a decimal ensuring place-value is respected
AnsaB := (V1 * 16) + V2                                                                 ' Convert into a decimal ensuring place-value is respected

pub callLCDShow(cTitle, cUnits, v)
{{callLCDShow is a procedure that displays the OBD live data to the user. Title text and units are included and it is able to display decimals and string values.}}

 AnsaA := v  
'if err == No                                                              ' Only display if recording is not in progress and there are no errors
   callCls                                                                              ' Clear the LCD
   debug.str(cTitle)                                                                      ' Show the title on the first row
   callPutC(58)                                                                         ' Add a colon
   debug.gotoXY(2,1)                                                                      ' Move the cursor to the next row
  
 '  if lcdMode == 0                                                                      ' Process the data depending on the value of lcdMode
     debug.dec(v)                                                                     ' Its a decimal
  ' else
   '  debug.str(v)                                                                     ' Its a string
   debug.str(cUnits)                                                                       ' Display the units
   
pub callPutC(x)
  debug.putc(x)

pub SDcomma
  sdfat.pputc(44)

pub SDCR
  sdfat.pputc(13) 
{{
                            TERMS OF USE: MIT License                                                           

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}  